@@ -265,6 +265,25 @@ public:
 
 struct Object;
 
+struct EnumValue : public QV4::CompiledData::EnumValue
+{
+    EnumValue *next;
+};
+
+struct Enum
+{
+    int nameIndex;
+    QV4::CompiledData::Location location;
+    PoolList<EnumValue> *enumValues;
+
+    int enumValueCount() const { return enumValues->count; }
+    PoolList<EnumValue>::Iterator enumValuesBegin() const { return enumValues->begin(); }
+    PoolList<EnumValue>::Iterator enumValuesEnd() const { return enumValues->end(); }
+
+    Enum *next;
+};
+
+
 struct SignalParameter : public QV4::CompiledData::Parameter
 {
     SignalParameter *next;
@@ -359,6 +378,8 @@ public:
     int propertyCount() const { return properties->count; }
     Alias *firstAlias() const { return aliases->first; }
     int aliasCount() const { return aliases->count; }
+    const Enum *firstEnum() const { return qmlEnums->first; }
+    int enumCount() const { return qmlEnums->count; }
     const Signal *firstSignal() const { return qmlSignals->first; }
     int signalCount() const { return qmlSignals->count; }
     Binding *firstBinding() const { return bindings->first; }
@@ -372,6 +393,8 @@ public:
     PoolList<Property>::Iterator propertiesEnd() const { return properties->end(); }
     PoolList<Alias>::Iterator aliasesBegin() const { return aliases->begin(); }
     PoolList<Alias>::Iterator aliasesEnd() const { return aliases->end(); }
+    PoolList<Enum>::Iterator enumsBegin() const { return qmlEnums->begin(); }
+    PoolList<Enum>::Iterator enumsEnd() const { return qmlEnums->end(); }
     PoolList<Signal>::Iterator signalsBegin() const { return qmlSignals->begin(); }
     PoolList<Signal>::Iterator signalsEnd() const { return qmlSignals->end(); }
     PoolList<Function>::Iterator functionsBegin() const { return functions->begin(); }
@@ -385,6 +408,7 @@ public:
 
     QString sanityCheckFunctionNames(const QSet<QString> &illegalNames, QQmlJS::AST::SourceLocation *errorLocation);
 
+    QString appendEnum(Enum *enumeration);
     QString appendSignal(Signal *signal);
     QString appendProperty(Property *prop, const QString &propertyName, bool isDefaultProperty, const QQmlJS::AST::SourceLocation &defaultToken, QQmlJS::AST::SourceLocation *errorLocation);
     QString appendAlias(Alias *prop, const QString &aliasName, bool isDefaultProperty, const QQmlJS::AST::SourceLocation &defaultToken, QQmlJS::AST::SourceLocation *errorLocation);
@@ -408,6 +432,7 @@ private:
 
     PoolList<Property> *properties;
     PoolList<Alias> *aliases;
+    PoolList<Enum> *qmlEnums;
     PoolList<Signal> *qmlSignals;
     PoolList<Binding> *bindings;
     PoolList<Function> *functions;
@@ -432,7 +457,6 @@ struct Q_QML_PRIVATE_EXPORT Document
     QList<const QV4::CompiledData::Import *> imports;
     QList<Pragma*> pragmas;
     QQmlJS::AST::UiProgram *program;
-    int indexOfRootObject;
     QVector<Object*> objects;
     QV4::Compiler::JSUnitGenerator jsGenerator;
 
@@ -453,9 +477,9 @@ struct Q_QML_PRIVATE_EXPORT ScriptDirectivesCollector : public QQmlJS::Directive
     QList<const QV4::CompiledData::Import *> imports;
     bool hasPragmaLibrary;
 
-    virtual void pragmaLibrary();
-    virtual void importFile(const QString &jsfile, const QString &module, int lineNumber, int column);
-    virtual void importModule(const QString &uri, const QString &version, const QString &module, int lineNumber, int column);
+    void pragmaLibrary() override;
+    void importFile(const QString &jsfile, const QString &module, int lineNumber, int column) override;
+    void importModule(const QString &uri, const QString &version, const QString &module, int lineNumber, int column) override;
 };
 
 struct Q_QML_PRIVATE_EXPORT IRBuilder : public QQmlJS::AST::Visitor
@@ -470,21 +494,22 @@ public:
     using QQmlJS::AST::Visitor::visit;
     using QQmlJS::AST::Visitor::endVisit;
 
-    virtual bool visit(QQmlJS::AST::UiArrayMemberList *ast);
-    virtual bool visit(QQmlJS::AST::UiImport *ast);
-    virtual bool visit(QQmlJS::AST::UiPragma *ast);
-    virtual bool visit(QQmlJS::AST::UiHeaderItemList *ast);
-    virtual bool visit(QQmlJS::AST::UiObjectInitializer *ast);
-    virtual bool visit(QQmlJS::AST::UiObjectMemberList *ast);
-    virtual bool visit(QQmlJS::AST::UiParameterList *ast);
-    virtual bool visit(QQmlJS::AST::UiProgram *);
-    virtual bool visit(QQmlJS::AST::UiQualifiedId *ast);
-    virtual bool visit(QQmlJS::AST::UiArrayBinding *ast);
-    virtual bool visit(QQmlJS::AST::UiObjectBinding *ast);
-    virtual bool visit(QQmlJS::AST::UiObjectDefinition *ast);
-    virtual bool visit(QQmlJS::AST::UiPublicMember *ast);
-    virtual bool visit(QQmlJS::AST::UiScriptBinding *ast);
-    virtual bool visit(QQmlJS::AST::UiSourceElement *ast);
+    bool visit(QQmlJS::AST::UiArrayMemberList *ast) override;
+    bool visit(QQmlJS::AST::UiImport *ast) override;
+    bool visit(QQmlJS::AST::UiPragma *ast) override;
+    bool visit(QQmlJS::AST::UiHeaderItemList *ast) override;
+    bool visit(QQmlJS::AST::UiObjectInitializer *ast) override;
+    bool visit(QQmlJS::AST::UiObjectMemberList *ast) override;
+    bool visit(QQmlJS::AST::UiParameterList *ast) override;
+    bool visit(QQmlJS::AST::UiProgram *) override;
+    bool visit(QQmlJS::AST::UiQualifiedId *ast) override;
+    bool visit(QQmlJS::AST::UiArrayBinding *ast) override;
+    bool visit(QQmlJS::AST::UiObjectBinding *ast) override;
+    bool visit(QQmlJS::AST::UiObjectDefinition *ast) override;
+    bool visit(QQmlJS::AST::UiEnumDeclaration *ast) override;
+    bool visit(QQmlJS::AST::UiPublicMember *ast) override;
+    bool visit(QQmlJS::AST::UiScriptBinding *ast) override;
+    bool visit(QQmlJS::AST::UiSourceElement *ast) override;
 
     void accept(QQmlJS::AST::Node *node);
 
@@ -548,7 +573,7 @@ public:
 
 struct Q_QML_PRIVATE_EXPORT QmlUnitGenerator
 {
-    QV4::CompiledData::Unit *generate(Document &output, QQmlEngine *engine, const QV4::CompiledData::ResolvedTypeReferenceMap &dependentTypes);
+    QV4::CompiledData::Unit *generate(Document &output, const QV4::CompiledData::DependentTypesHasher &dependencyHasher = QV4::CompiledData::DependentTypesHasher());
 
 private:
     typedef bool (Binding::*BindingFilter)() const;
@@ -602,8 +627,8 @@ struct Q_QML_PRIVATE_EXPORT JSCodeGen : public QQmlJS::Codegen
     QVector<int> generateJSCodeForFunctionsAndBindings(const QList<CompiledFunctionOrExpression> &functions);
 
 protected:
-    virtual void beginFunctionBodyHook();
-    virtual QV4::IR::Expr *fallbackNameLookup(const QString &name, int line, int col);
+    void beginFunctionBodyHook() override;
+    QV4::IR::Expr *fallbackNameLookup(const QString &name, int line, int col) override;
 
 private:
     QQmlPropertyData *lookupQmlCompliantProperty(QQmlPropertyCache *cache, const QString &name, bool *propertyExistsButForceNameLookup = 0);
@@ -622,6 +647,21 @@ private:
     int _importedScriptsTemp;
 };
 
+struct Q_QML_PRIVATE_EXPORT IRLoader {
+    IRLoader(const QV4::CompiledData::Unit *unit, QmlIR::Document *output);
+
+    void load();
+
+private:
+    QmlIR::Object *loadObject(const QV4::CompiledData::Object *serializedObject);
+
+    template <typename _Tp> _Tp *New() { return pool->New<_Tp>(); }
+
+    const QV4::CompiledData::Unit *unit;
+    QmlIR::Document *output;
+    QQmlJS::MemoryPool *pool;
+};
+
 } // namespace QmlIR
 
 QT_END_NAMESPACE
