@@ -59,29 +59,41 @@ namespace QV4 {
 
 namespace Heap {
 
-struct MemberData : Base {
-    union {
-        uint size;
-        double _dummy;
-    };
-    Value data[1];
+#define MemberDataMembers(class, Member) \
+    Member(class, ValueArray, ValueArray, values)
+
+DECLARE_HEAP_OBJECT(MemberData, Base) {
+    DECLARE_MARK_TABLE(MemberData);
 };
+V4_ASSERT_IS_TRIVIAL(MemberData)
 
 }
 
 struct MemberData : Managed
 {
     V4_MANAGED(MemberData, Managed)
+    V4_INTERNALCLASS(MemberData)
 
-    Value &operator[] (uint idx) { return d()->data[idx]; }
-    const Value *data() const { return d()->data; }
-    Value *data() { return d()->data; }
-    inline uint size() const { return d()->size; }
+    struct Index {
+        Heap::Base *base;
+        Value *slot;
 
-    static Heap::MemberData *allocate(QV4::ExecutionEngine *e, uint n);
-    static Heap::MemberData *reallocate(QV4::ExecutionEngine *e, Heap::MemberData *old, uint n);
+        void set(EngineBase *e, Value newVal) {
+            WriteBarrier::write(e, base, slot, newVal);
+        }
+        const Value *operator->() const { return slot; }
+        const Value &operator*() const { return *slot; }
+        bool isNull() const { return !slot; }
+    };
 
-    static void markObjects(Heap::Base *that, ExecutionEngine *e);
+    const Value &operator[] (uint idx) const { return d()->values[idx]; }
+    const Value *data() const { return d()->values.data(); }
+    void set(EngineBase *e, uint index, Value v) { d()->values.set(e, index, v); }
+    void set(EngineBase *e, uint index, Heap::Base *b) { d()->values.set(e, index, b); }
+
+    inline uint size() const { return d()->values.size; }
+
+    static Heap::MemberData *allocate(QV4::ExecutionEngine *e, uint n, Heap::MemberData *old = 0);
 };
 
 }
